@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Alberts Muktupāvels
+ * Copyright (C) 2014-2017 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,12 @@
 #include <gtk/gtk.h>
 
 #include "gf-application.h"
+#include "gf-confirm-display-change-dialog.h"
+#include "backends/gf-backend.h"
 #include "libaudio-device-selection/gf-audio-device-selection.h"
 #include "libautomount-manager/gsd-automount-manager.h"
 #include "libbluetooth-applet/gf-bluetooth-applet.h"
 #include "libdesktop-background/gf-desktop-background.h"
-#include "libdisplay-config/flashback-display-config.h"
 #include "libend-session-dialog/gf-end-session-dialog.h"
 #include "libidle-monitor/flashback-idle-monitor.h"
 #include "libinput-settings/gf-input-settings.h"
@@ -38,44 +39,41 @@
 #include "libscreenshot/gf-screenshot.h"
 #include "libshell/flashback-shell.h"
 #include "libsound-applet/gf-sound-applet.h"
+#include "libstatus-notifier-watcher/gf-status-notifier-watcher.h"
 #include "libworkarounds/gf-workarounds.h"
 
-#ifdef WITH_LIBSTATUS_NOTIFIER
-#include <libstatus-notifier/sn.h>
-#endif
-
 struct _GfApplication
 {
-  GObject                 parent;
+  GObject                  parent;
 
-  gint                    bus_name;
+  GfBackend               *backend;
 
-  GSettings              *settings;
+  gint                     bus_name;
 
-  GtkStyleProvider       *provider;
+  GSettings               *settings;
 
-  GsdAutomountManager    *automount;
-  FlashbackDisplayConfig *config;
-  FlashbackIdleMonitor   *idle_monitor;
-  FlashbackPolkit        *polkit;
-  FlashbackShell         *shell;
-  GfAudioDeviceSelection *audio_device_selection;
-  GfBluetoothApplet      *bluetooth;
-  GfDesktopBackground    *background;
-  GfEndSessionDialog     *dialog;
-  GfInputSettings        *input_settings;
-  GfInputSources         *input_sources;
-  GfNotifications        *notifications;
-  GfPowerApplet          *power;
-  GfScreencast           *screencast;
-  GfScreensaver          *screensaver;
-  GfScreenshot           *screenshot;
-  GfSoundApplet          *sound;
-  GfWorkarounds          *workarounds;
+  GtkStyleProvider        *provider;
 
-#ifdef WITH_LIBSTATUS_NOTIFIER
-  SnWatcher              *watcher;
-#endif
+  GsdAutomountManager     *automount;
+  FlashbackIdleMonitor    *idle_monitor;
+  FlashbackPolkit         *polkit;
+  FlashbackShell          *shell;
+  GfAudioDeviceSelection  *audio_device_selection;
+  GfBluetoothApplet       *bluetooth;
+  GfDesktopBackground     *background;
+  GfEndSessionDialog      *dialog;
+  GfInputSettings         *input_settings;
+  GfInputSources          *input_sources;
+  GfNotifications         *notifications;
+  GfPowerApplet           *power;
+  GfScreencast            *screencast;
+  GfScreensaver           *screensaver;
+  GfScreenshot            *screenshot;
+  GfSoundApplet           *sound;
+  GfStatusNotifierWatcher *status_notifier_watcher;
+  GfWorkarounds           *workarounds;
+
+  GtkWidget               *display_change_dialog;
 };
 
 G_DEFINE_TYPE (GfApplication, gf_application, G_TYPE_OBJECT)
@@ -136,8 +134,10 @@ settings_changed (GSettings   *settings,
                   gpointer     user_data)
 {
   GfApplication *application;
+  GfMonitorManager *monitor_manager;
 
   application = GF_APPLICATION (user_data);
+  monitor_manager = gf_backend_get_monitor_manager (application->backend);
 
 #define SETTING_CHANGED(variable_name, setting_name, function_name) \
   if (key == NULL || g_strcmp0 (key, setting_name) == 0)            \
@@ -154,7 +154,6 @@ settings_changed (GSettings   *settings,
     }
 
   SETTING_CHANGED (automount, "automount-manager", gsd_automount_manager_new)
-  SETTING_CHANGED (config, "display-config", flashback_display_config_new)
   SETTING_CHANGED (idle_monitor, "idle-monitor", flashback_idle_monitor_new)
   SETTING_CHANGED (polkit, "polkit", flashback_polkit_new)
   SETTING_CHANGED (shell, "shell", flashback_shell_new)
@@ -170,31 +169,17 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (screensaver, "screensaver", gf_screensaver_new)
   SETTING_CHANGED (screenshot, "screenshot", gf_screenshot_new)
   SETTING_CHANGED (sound, "sound-applet", gf_sound_applet_new)
+  SETTING_CHANGED (status_notifier_watcher, "status-notifier-watcher", gf_status_notifier_watcher_new)
   SETTING_CHANGED (workarounds, "workarounds", gf_workarounds_new)
 
 #undef SETTING_CHANGED
 
-#ifdef WITH_LIBSTATUS_NOTIFIER
-  if (key == NULL || g_strcmp0 (key, "watcher") == 0)
-    {
-      if (g_settings_get_boolean (settings, "watcher"))
-        {
-          if (application->watcher == NULL)
-            application->watcher = sn_watcher_new (SN_WATCHER_FLAGS_NONE);
-        }
-      else
-        {
-          g_clear_object (&application->watcher);
-        }
-    }
-#endif
-
   if (application->input_settings)
-    gf_input_settings_set_display_config (application->input_settings,
-                                          application->config);
+    gf_input_settings_set_monitor_manager (application->input_settings,
+                                           monitor_manager);
 
   if (application->shell)
-    flashback_shell_set_display_config (application->shell, application->config);
+    flashback_shell_set_monitor_manager (application->shell, monitor_manager);
 }
 
 static void
@@ -215,7 +200,6 @@ gf_application_dispose (GObject *object)
   g_clear_object (&application->provider);
 
   g_clear_object (&application->automount);
-  g_clear_object (&application->config);
   g_clear_object (&application->idle_monitor);
   g_clear_object (&application->polkit);
   g_clear_object (&application->shell);
@@ -230,19 +214,57 @@ gf_application_dispose (GObject *object)
   g_clear_object (&application->screenshot);
   g_clear_object (&application->screensaver);
   g_clear_object (&application->sound);
+  g_clear_object (&application->status_notifier_watcher);
   g_clear_object (&application->workarounds);
 
-#ifdef WITH_LIBSTATUS_NOTIFIER
-  g_clear_object (&application->watcher);
-#endif
+  g_clear_pointer (&application->display_change_dialog, gtk_widget_destroy);
+
+  g_clear_object (&application->backend);
 
   G_OBJECT_CLASS (gf_application_parent_class)->dispose (object);
 }
 
+static void
+keep_changes_cb (GfConfirmDisplayChangeDialog *dialog,
+                 gboolean                      keep_changes,
+                 GfApplication                *application)
+{
+  GfMonitorManager *monitor_manager;
+
+  monitor_manager = gf_backend_get_monitor_manager (application->backend);
+
+  gf_monitor_manager_confirm_configuration (monitor_manager, keep_changes);
+  g_clear_pointer (&application->display_change_dialog, gtk_widget_destroy);
+}
+
+static void
+confirm_display_change_cb (GfMonitorManager *monitor_manager,
+                           GfApplication    *application)
+{
+  gint timeout;
+
+  timeout = gf_monitor_manager_get_display_configuration_timeout ();
+
+  g_clear_pointer (&application->display_change_dialog, gtk_widget_destroy);
+  application->display_change_dialog = gf_confirm_display_change_dialog_new (timeout);
+
+  g_signal_connect (application->display_change_dialog, "keep-changes",
+                    G_CALLBACK (keep_changes_cb), application);
+
+  gtk_window_present (GTK_WINDOW (application->display_change_dialog));
+}
+
 static void
 gf_application_init (GfApplication *application)
 {
   GtkSettings *settings;
+  GfMonitorManager *monitor_manager;
+
+  application->backend = gf_backend_new (GF_BACKEND_TYPE_X11_CM);
+
+  monitor_manager = gf_backend_get_monitor_manager (application->backend);
+  g_signal_connect (monitor_manager, "confirm-display-change",
+                    G_CALLBACK (confirm_display_change_cb), application);
 
   application->settings = g_settings_new ("org.gnome.gnome-flashback");
   settings = gtk_settings_get_default ();
