@@ -47,6 +47,7 @@ typedef struct
   gboolean        url_clicked_lock;
 
   guint           timeout_id;
+  gulong          changed_id;
 } GfBubblePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GfBubble, gf_bubble, GF_TYPE_POPUP_WINDOW)
@@ -353,6 +354,7 @@ update_bubble (GfBubble *bubble)
   /* Icon */
 
   pixbuf = nd_notification_load_image (priv->notification, IMAGE_SIZE);
+  have_icon = FALSE;
 
   if (pixbuf != NULL)
     {
@@ -413,6 +415,12 @@ gf_bubble_dispose (GObject *object)
       priv->timeout_id = 0;
     }
 
+  if (priv->changed_id != 0)
+    {
+      g_signal_handler_disconnect (priv->notification, priv->changed_id);
+      priv->changed_id = 0;
+    }
+
   G_OBJECT_CLASS (gf_bubble_parent_class)->dispose (object);
 }
 
@@ -591,7 +599,7 @@ gf_bubble_init (GfBubble *bubble)
   atk_action_set_description (ATK_ACTION (atkobj), 0,
                               _("Closes the notification."));
 
-  image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
+  image = gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (priv->close_button), image);
   gtk_widget_show (image);
 
@@ -660,14 +668,15 @@ gf_bubble_new_for_notification (NdNotification *notification)
   bubble = g_object_new (GF_TYPE_BUBBLE,
                          "resizable", FALSE,
                          "title", _("Notification"),
+                         "type", GTK_WINDOW_POPUP,
                          NULL);
 
   priv = gf_bubble_get_instance_private (bubble);
 
   priv->notification = g_object_ref (notification);
-
-  g_signal_connect (notification, "changed",
-                    G_CALLBACK (notification_changed_cb), bubble);
+  priv->changed_id = g_signal_connect (notification, "changed",
+                                       G_CALLBACK (notification_changed_cb),
+                                       bubble);
 
   update_bubble (bubble);
 
