@@ -25,6 +25,8 @@ struct _FlashbackOsd
 
   GfOsdWindow **windows;
   gint          n_monitors;
+
+  gulong        monitors_changed_id;
 };
 
 G_DEFINE_TYPE (FlashbackOsd, flashback_osd, G_TYPE_OBJECT)
@@ -34,11 +36,14 @@ monitors_changed (GdkScreen *screen,
                   gpointer   user_data)
 {
   FlashbackOsd *osd;
+  GdkDisplay *display;
   gint n_monitors;
   gint i;
 
   osd = FLASHBACK_OSD (user_data);
-  n_monitors = gdk_screen_get_n_monitors (screen);
+
+  display = gdk_display_get_default ();
+  n_monitors = gdk_display_get_n_monitors (display);
 
   if (osd->windows != NULL)
     {
@@ -64,7 +69,16 @@ flashabck_osd_finalize (GObject *object)
   screen = gdk_screen_get_default ();
   osd = FLASHBACK_OSD (object);
 
-  g_signal_handlers_disconnect_by_func (screen, monitors_changed, osd);
+  g_signal_handler_disconnect (screen, osd->monitors_changed_id);
+
+  if (osd->windows != NULL)
+    {
+      gint i;
+
+      for (i = 0; i < osd->n_monitors; i++)
+        gtk_widget_destroy (GTK_WIDGET (osd->windows[i]));
+      g_free (osd->windows);
+    }
 
   G_OBJECT_CLASS (flashback_osd_parent_class)->finalize (object);
 }
@@ -86,8 +100,9 @@ flashback_osd_init (FlashbackOsd *osd)
 
   screen = gdk_screen_get_default ();
 
-  g_signal_connect (screen, "monitors-changed",
-                    G_CALLBACK (monitors_changed), osd);
+  osd->monitors_changed_id =
+    g_signal_connect (screen, "monitors-changed",
+                      G_CALLBACK (monitors_changed), osd);
 
   monitors_changed (screen, osd);
 }
